﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;

using OldBvets.TrainDat;
using BvetsTool;

namespace WindowsFormsApplication1
{
    using Bvets5.Vehicle;
    using BvetsToolApi;


    public partial class AccelerateCurveForm : Form
    {
        bool onChangingNotch = false; // ノッチ変更によるパラメータ数値を変更中か否か
        DrawGraph.ValueType valueType; // Y軸数値の種類
        Bitmap graphBitmap; // グラフ用ビットマップ
        DrawGraph drawCurve; // グラフ描画オブジェクト
        Acceleration[] accParam; // 加速力曲線パラメーター
        Point[][] point; // 座標[インデックス, 点群]
        Pen[] pen; // グラフ描画用ペン

        Color editerModeBgCol;
        Color viewerModeBgCol;

        double[][,] forceData; // 加速力テーブル格納
        double[][,] currentData; // 電流テーブル格納

        int cnt = 0;
        double colorRate;

        public AccelerateCurveForm()
        {
            InitializeComponent();
        }

        // ******************** 初期化 ******************** 
        // 初期化(Load)
        private void AccelerateCurveForm_Load(object sender, EventArgs e)
        {
            // ノッチ情報格納用オブジェクト作成
            accParam = new Acceleration[8];
            for (int i = 0; i < 8; i++)
            {
                if (i == 2)
                {
                    accParam[i] = new Acceleration(3.3, 3.3, 40.0, 40.0, 2.0);
                }
                else if (i == 3)
                {
                    accParam[i] = new Acceleration(3.3, 3.3, 40, 55, 2);
                }
                else if (i == 4)
                {
                    accParam[i] = new Acceleration(3.3, 3.3, 40, 70, 2);
                }
                else if (i == 0)
                {
                    accParam[i] = new Acceleration(1, 1, 15, 15, 2);
                }
                else if (i == 1)
                {
                    accParam[i] = new Acceleration(3.3, 3.3, 20, 20, 2);
                }
                else
                {
                    accParam[i] = new Acceleration(0, 0, 0, 0, 1);
                }
            }

            // パラメータの初期化
            Bvets5.Vehicle.Parameters.Dynamics.Initialize();

            // 初期曲線タイプの選択
            valueType = DrawGraph.ValueType.Acceleration;

            // 非選択インデックスの曲線色
            colorRate = 0.5;

            // 背景色
            editerModeBgCol = Color.Black;
            viewerModeBgCol = Color.FromArgb(0, 0, 48);
        }

        // 初期化(Shown)
        private void AccelerateCurveForm_Shown(object sender, EventArgs e)
        {

            graphBitmap = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            drawCurve = new DrawGraph(graphBitmap);
            drawCurve.BackGroundColor = Color.FromArgb(0, 0, 0);

            masconNumber.SelectedIndex = 0;

            // checkBoxSubResistance.Checked = true;
        }

        // ******************** 汎用メソッド群 ******************** 

        // グラフ更新
        private void refleshImage()
        {
            cnt++;
            // this.Text = cnt.ToString();

            if (editModeTsmi.Checked)
            {
                // 座標数値計算
                point = drawCurve.CalcPoints(accParam, valueType, checkBoxSubResistance.Checked);

                // グリッド・数値描画
                drawCurve.DrawBase(graphBitmap);

                // ペン色設定
                pen = drawCurve.SetPenColor(masconNumber.SelectedIndex, checkBoxSubResistance.Checked, colorRate);

                // 曲線描画
                drawCurve.DrawCurves(pen, point, masconNumber.SelectedIndex);

                // PictureBoxを更新
                pictureBox1.Image = graphBitmap;
            }
            else
            {
                // グリッド・数値描画
                drawCurve.DrawBase(graphBitmap);

                // 座標数値計算
                switch (valueType)
                {
                    case DrawGraph.ValueType.Acceleration:
                    case DrawGraph.ValueType.Force:
                        if (forceData != null)
                        {
                            point = drawCurve.CalcPoints(forceData, valueType, checkBoxSubResistance.Checked);
                        }
                        else
                        {
                            pictureBox1.Image = graphBitmap;
                            return;
                        }
                        break;
                    case DrawGraph.ValueType.Current:
                        if (currentData != null)
                        {
                            point = drawCurve.CalcPoints(currentData, valueType, checkBoxSubResistance.Checked);
                        }
                        else
                        {
                            pictureBox1.Image = graphBitmap;
                            return;
                        }
                        break;
                }
                
                // ペン色設定
                pen = drawCurve.SetPenColor(masconNumber.SelectedIndex, checkBoxSubResistance.Checked, colorRate);

                // 曲線描画
                drawCurve.DrawCurves(pen, point, masconNumber.SelectedIndex);

                // PictureBoxを更新
                pictureBox1.Image = graphBitmap;
            }
        }

        // ドラッグドロップによる読み込み
        private void AccelerateCurveForm_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {

                // ドラッグ中のファイルやディレクトリの取得
                string[] drags = (string[])e.Data.GetData(DataFormats.FileDrop);

                foreach (string d in drags)
                {
                    if (!System.IO.File.Exists(d))
                    {
                        return;// ファイル以外のDDであればイベント・ハンドラを抜ける
                    }
                }
                e.Effect = DragDropEffects.Copy;
            }
        }

        private void AccelerateCurveForm_DragDrop(object sender, DragEventArgs e)
        {
            // ドラッグ＆ドロップされたファイル
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            string a = Path.GetExtension(files[0]);

            // datファイル
            if (String.Compare(Path.GetExtension(files[0]), ".dat", true) == 0)
            {
                OpenFile.InportTraindat(files[0], accParam);
                setTrainDatParams(true);
                refleshImage();
            }
            // csvファイル
            else if (String.Compare(Path.GetExtension(files[0]), ".csv", true) == 0)
            {
                openCsv(files[0]);
            }
            // txt、cfgファイル
            else if (String.Compare(Path.GetExtension(files[0]), ".txt", true) == 0 || String.Compare(Path.GetExtension(files[0]), ".cfg", true) == 0)
            {
                OpenFile.OpenText(files[0]);
                setTrainDatParams(false);
            }
            // その他
            else
            {
                MessageBox.Show("読み込めない形式です。",
                    "警告",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }

        }

        // インデックスを切り替えたときのノッチパラメータのコントロール値変更
        private void changeNotchParam()
        {
            if (editModeTsmi.Checked & masconNumber.SelectedIndex >= 0) // エディタモードかつコンボボックスが空でない場合
            {
                onChangingNotch = true;
                int a = masconNumber.SelectedIndex;
                constAccelerationStart.Value = (decimal)accParam[masconNumber.SelectedIndex].ConstAccelerationStart;
                constAccelerationEnd.Value = (decimal)accParam[masconNumber.SelectedIndex].ConstAccelerationEnd;
                constPowerStart.Value = (decimal)accParam[masconNumber.SelectedIndex].ConstPowerStart;
                constPowerEnd.Value = (decimal)accParam[masconNumber.SelectedIndex].ConstPowerEnd;
                characteristicCurveDegree.Value = (decimal)accParam[masconNumber.SelectedIndex].CharacteristicCurveDegree;
                onChangingNotch = false;
            }
        }

        // 読み込んだTrain.dat/Parametersのデータをセット
        private void setTrainDatParams(bool traindat)
        {

            // 共通
            try
            {
                motorcarWeight.Value = (decimal)Parameters.Dynamics.MotorcarWeight / 1000;
            }
            catch { }
            try
            {
                motorcarCount.Value = (decimal)Parameters.Dynamics.MotorcarCount;
            }
            catch { }
            try
            {
                trailerWeight.Value = (decimal)Parameters.Dynamics.TrailerWeight / 1000;
            }
            catch { }
            try
            {
                trailerCount.Value = (decimal)Parameters.Dynamics.TrailerCount;
            }
            catch { }

            if (traindat) // Train.datのみ
            {
                changeNotchParam();

                try
                {
                    if (Parameters.Original.LoadCompensating != 0)
                    {
                        loadCompensating.Value = (decimal)Parameters.Original.LoadCompensating;
                    }
                }
                catch { }
            }
            else // Parametersのみ
            {
                try
                {
                    if (Parameters.Dynamics.MotorcarInertiaFactor != 0)
                    {
                        motorcarInatiaFactor.Value = (decimal)Parameters.Dynamics.MotorcarInertiaFactor;
                    }
                }
                catch { }
                try
                {
                    if (Parameters.Dynamics.TrailerInertiaFactor != 0)
                    {
                        trailerInatiaFactor.Value = (decimal)Parameters.Dynamics.TrailerInertiaFactor;
                    }
                }
                catch { }
            }
        }

        // CSVファイルを開く
        private void openCsv(string filename)
        {
            if (editModeTsmi.Checked)
            {
                toViewerMode(); // ビューワモードヘ
            }

            double[][,] table;
            table = OpenFile.CsvFileParser(filename);

            if (table == null)
            {
                return;
            }

            // 以下二文をここに置くとインデックス数の
            // 減少時にchangeConboBox()以降が読み込まれない
            // drawCurve.MaxNotch = table.GetLength(0);
            // changeComboBox();

            // ForceとCurrentのデータ振り分け（ファイル名で振り分け、判別不能時にダイアログ表示）
            if (System.IO.Path.GetFileName(filename).ToLower().Contains("current"))
            {
                currentData = null;
                currentData = table;
                radioButtonCurrent.Checked = true;
            }
            else if (System.IO.Path.GetFileName(filename).ToLower().Contains("force"))
            {
                forceData = null;
                forceData = table;
                radioButtonForce.Checked = true;
            }
            else
            {
                SelectValueType svl = new SelectValueType();
                svl.ShowDialog();
                DialogResult force = svl.DialogResult;
                if (force == DialogResult.Yes)
                {
                    forceData = null;
                    forceData = table;
                    radioButtonForce.Checked = true;
                }
                else if (force == DialogResult.No)
                {
                    currentData = null;
                    currentData = table;
                    radioButtonCurrent.Checked = true;
                }
                
            }

            drawCurve.MaxNotch = table.GetLength(0);
            changeComboBox();

            refleshImage();
        }

        // エディタモードへの切り替え
        private void toEditerMode()
        {

            editModeTsmi.Checked = true;
            viewerModeTsmi.Checked = false;

            drawCurve.BackGroundColor = editerModeBgCol;

            drawCurve.MaxNotch = 8;
            changeComboBox();

            this.Text = "Notch Curve Editor";
            changeNotchParam();
            constAccelerationStart.Enabled = true;
            constAccelerationEnd.Enabled = true;
            syncAccelerate.Enabled = true;
            constPowerStart.Enabled = true;
            constPowerEnd.Enabled = true;
            characteristicCurveDegree.Enabled = true;
            copyNotchCurve.Enabled = true;

            radioButtonCurrent.Text = "相対電流";
            if (radioButtonCurrent.Checked)
            {
                drawCurve.YMax = 140;
                drawCurve.YInterval = 20;
            }

            exportTableTsmi.Enabled = true;

            refleshImage();
        }

        // ビューワモードへの切り替え
        private void toViewerMode()
        {
            editModeTsmi.Checked = false;
            viewerModeTsmi.Checked = true;

            drawCurve.BackGroundColor = viewerModeBgCol;

            if (radioButtonAcceleration.Checked | radioButtonForce.Checked)
            {
                if (forceData != null)
                {
                    drawCurve.MaxNotch = forceData.Length;
                    changeComboBox();
                }
            }
            else
            {
                if (currentData != null)
                {
                    drawCurve.MaxNotch = currentData.Length;
                    changeComboBox();
                }
            }
            this.Text = "Notch Curve Viewer";
            constAccelerationStart.Enabled = false;
            constAccelerationEnd.Enabled = false;
            syncAccelerate.Enabled = false;
            constPowerStart.Enabled = false;
            constPowerEnd.Enabled = false;
            characteristicCurveDegree.Enabled = false;
            copyNotchCurve.Enabled = false;

            radioButtonCurrent.Text = "電流[A]";
            if (radioButtonCurrent.Checked)
            {
                drawCurve.YMax = 800;
                drawCurve.YInterval = 100;
            }

            exportTableTsmi.Enabled = false;
            refleshImage();
        }

        // コンボボックスのインデックス数更新
        private void changeComboBox()
        {
            string[] itemObject = new string[drawCurve.MaxNotch];
            for (int i = 0; i < drawCurve.MaxNotch; i++)
            {
                itemObject[i] = (i + 1).ToString();
            }
            masconNumber.Items.Clear();
            masconNumber.Items.AddRange(itemObject);
            masconNumber.SelectedIndex = 0;
        }
        // ******************** コントロール ******************** 

        // ノッチインデックス変更
        private void masconNumber_SelectedIndexChanged(object sender, EventArgs e)
        {
            changeNotchParam();
            refleshImage();
        }


        // 定引張力域開始
        private void constAccelerationStart_ValueChanged(object sender, EventArgs e)
        {
            if (syncAccelerate.Checked & !onChangingNotch)
            {
                constAccelerationEnd.Value = constAccelerationStart.Value;
            }
            accParam[masconNumber.SelectedIndex].ConstAccelerationStart = (double)constAccelerationStart.Value;
            refleshImage();
        }

        // 定引張力域終了
        private void constAccelerationEnd_ValueChanged(object sender, EventArgs e)
        {
            if (syncAccelerate.Checked & !onChangingNotch)
            {
                constAccelerationStart.Value = constAccelerationEnd.Value;
            }
            accParam[masconNumber.SelectedIndex].ConstAccelerationEnd = (double)constAccelerationEnd.Value;
            refleshImage();
        }

        // 定引張力域同期
        private void syncAccelerate_CheckedChanged(object sender, EventArgs e)
        {
            if (syncAccelerate.Checked)
            {
                constAccelerationStart.Value = constAccelerationEnd.Value;
            }
        }

        // 定出力域開始
        private void constPowerStart_ValueChanged(object sender, EventArgs e)
        {
            // ノッチインデックス変更中は数値修正オフ
            if (constPowerStart.Value > constPowerEnd.Value & !onChangingNotch)
            {
                constPowerEnd.Value = constPowerStart.Value;
            }
            
            accParam[masconNumber.SelectedIndex].ConstPowerStart = (double)constPowerStart.Value;
            refleshImage();
        }

        // 定出力域終了
        private void constPowerEnd_ValueChanged(object sender, EventArgs e)
        {
            // ノッチインデックス変更中は数値修正オフ
            if (constPowerEnd.Value < constPowerStart.Value & !onChangingNotch)
            {
                constPowerStart.Value = constPowerEnd.Value;
            }
            accParam[masconNumber.SelectedIndex].ConstPowerEnd = (double)constPowerEnd.Value;
            refleshImage();
        }

        // 特性域
        private void characteristicCurveDegree_ValueChanged(object sender, EventArgs e)
        {
            accParam[masconNumber.SelectedIndex].CharacteristicCurveDegree = (double)characteristicCurveDegree.Value;
            refleshImage();
        }

        // 次のIndexにコピー
        private void copyNotchCurve_Click(object sender, EventArgs e)
        {

            if (masconNumber.SelectedIndex < drawCurve.MaxNotch - 1)
            {
                accParam[masconNumber.SelectedIndex + 1] = accParam[masconNumber.SelectedIndex];
                masconNumber.SelectedIndex = masconNumber.SelectedIndex + 1;
            }
            refleshImage();
        }

        // 曲線種類の選択
        private void radioButtonAcceleration_CheckedChanged(object sender, EventArgs e)
        {
            valueType = DrawGraph.ValueType.Acceleration;

            drawCurve.YMax = 4;
            drawCurve.YInterval = 1;

            if (!editModeTsmi.Checked & forceData != null)
            {
                drawCurve.MaxNotch = forceData.GetLength(0);
                changeComboBox();
            }
            refleshImage();
        }

        private void radioButtonForce_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonForce.Checked)
            {
                valueType = DrawGraph.ValueType.Force;

                drawCurve.YMax = 100000;
                drawCurve.YInterval = 10000;

                if (!editModeTsmi.Checked & forceData != null)
                {
                    drawCurve.MaxNotch = forceData.GetLength(0);
                    changeComboBox();
                }
                refleshImage();
            }
        }

        private void radioButtonCurrent_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonCurrent.Checked)
            {
                valueType = DrawGraph.ValueType.Current;
                if (editModeTsmi.Checked)
                {
                    drawCurve.YMax = 140;
                    drawCurve.YInterval = 20;
                }
                else
                {
                    drawCurve.YMax = 800;
                    drawCurve.YInterval = 100;
                    if (currentData != null)
                    {
                        drawCurve.MaxNotch = currentData.GetLength(0);
                        changeComboBox();
                    }
                }
                refleshImage();
            }
        }

        // 走行抵抗を引く
        private void checkBoxSubResistance_CheckedChanged(object sender, EventArgs e)
        {
            refleshImage();
        }

        // X軸(速度)スケール変更
        private void checkBoxScaleChange_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxScaleChange.Checked)
            {
                drawCurve.XMax = 320;
                drawCurve.XInterval = 20;
            }
            else
            {
                drawCurve.XMax = 160;
                drawCurve.XInterval = 10;
            }
            refleshImage();
        }

        // グラフ上の実数値の座標表示
        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {

            PointInfo.Text = string.Format(" {0}, {1}", drawCurve.PointX(e.X), drawCurve.PointY(e.Y));
        }
        
        // 走行抵抗係数の自動設定チェック
        private void checkBoxDefaultResistance_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxDefaultResistance.Checked)
            {
                setResistanceFactor();

                runningResistanceA.Enabled = false;
                runningResistanceB.Enabled = false;
                runningResistanceC.Enabled = false;
            }
            else
            {
                runningResistanceA.Enabled = true;
                runningResistanceB.Enabled = true;
                runningResistanceC.Enabled = true;
            }
        }

        // 走行抵抗係数入力
        // a
        private void runningResistanceA_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.RunningResistanceFactorA = (double)runningResistanceA.Value;
            // 自動係数設定時の多重描画防止
            if (!checkBoxDefaultResistance.Checked)
            {
                refleshImage();
            }
        }

        // b
        private void runningResistanceB_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.RunningResistanceFactorB = (double)runningResistanceB.Value;
            if (!checkBoxDefaultResistance.Checked)
            {
                refleshImage();
            }
        }

        // c
        private void runningResistanceC_ValueChanged_1(object sender, EventArgs e)
        {
            Parameters.Dynamics.RunningResistanceFactorC = (double)runningResistanceC.Value;
            if (!checkBoxDefaultResistance.Checked)
            {
                refleshImage();
            }
        }

        // 車両物理量入力
        // M車質量
        private void motorcarWeight_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.MotorcarWeight = 1000 * (double)motorcarWeight.Value; // [t]から[kg]へ
            setResistanceFactor();
            refleshImage();
        }

        // M車両数
        private void motorcarCount_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.MotorcarCount = (double)motorcarCount.Value;
            setResistanceFactor();
            refleshImage();
        }

        // T車質量
        private void trailerWeight_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.TrailerWeight = 1000 * (double)trailerWeight.Value; // [t]から[kg]へ
            setResistanceFactor();
            refleshImage();
        }

        // T車両数
        private void trailerCount_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.TrailerCount = (double)trailerCount.Value;
            setResistanceFactor();
            refleshImage();
        }

        // 主電動機個数(未使用)
        private void mainmotorCount_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Original.MainmotorCount = (double)mainmotorCount.Value;
        }

        // M車慣性係数
        private void motorcarInatiaFactor_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.MotorcarInertiaFactor = (double)motorcarInatiaFactor.Value;
            OrigMath.CalcRunningResistanceFactor();
            refleshImage();
        }

        // T車慣性係数
        private void trailerInatiaFactor_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Dynamics.TrailerInertiaFactor = (double)trailerInatiaFactor.Value;
            OrigMath.CalcRunningResistanceFactor();
            refleshImage();
        }

        // 応荷重
        private void loadCompensating_ValueChanged(object sender, EventArgs e)
        {
            Parameters.Original.LoadCompensating = (double)loadCompensating.Value * 1000;
            refleshImage();
        }


        // 走行抵抗係数の上書き
        private void setResistanceFactor()
        {
            OrigMath.CalcRunningResistanceFactor();

            if (checkBoxDefaultResistance.Checked)
            {
                // Numericupdownは範囲外Valueの代入で例外になる
                try
                {
                    runningResistanceA.Value = (decimal)Parameters.Dynamics.RunningResistanceFactorA;
                    runningResistanceB.Value = (decimal)Parameters.Dynamics.RunningResistanceFactorB;
                    runningResistanceC.Value = (decimal)Parameters.Dynamics.RunningResistanceFactorC;
                }
                catch
                {
                }
            }
        }

        // ******************** メニュー ******************** 

        // ファイル
        // Train.datインポート
        private void openTrainDatTsmi_Click(object sender, EventArgs e)
        {
            if (openTraindatDialog.ShowDialog() == DialogResult.OK)
            {
                toEditerMode(); // エディタモードへ

                OpenFile.InportTraindat(openTraindatDialog.FileName, accParam);

                setTrainDatParams(true);

                refleshImage();
            }
        }

        // CSVファイルを開く
        private void openCsvTsmi_Click(object sender, EventArgs e)
        {
            if (openCsvDialog.ShowDialog() == DialogResult.OK)
            {
                openCsv(openCsvDialog.FileName);
            }
        }

        // CSV「エクスポート」
        private void exportTable_Click(object sender, EventArgs e)
        {
            // フォルダ選択ダイアログ
            saveFolderBrowserDialog.SelectedPath = Directory.GetCurrentDirectory();
            DialogResult dr = saveFolderBrowserDialog.ShowDialog();

            if (dr == DialogResult.OK)
            {
                /*
                if (!Directory.Exists(saveFolderBrowserDialog.SelectedPath))
                {
                    // 選択ディレクトリが存在しない場合
                    MessageBox.Show("選択されたディレクトリは存在しません。",
                       "警告",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Exclamation);
                    return;
                }
                */

                string filepath = saveFolderBrowserDialog.SelectedPath;
                double[,] force;
                double[,] current;
                double[,] maxForce;
                double[,] maxCurrent;
                string csv;

                // Force.csv
                force = OrigMath.CreateForceTable(accParam, 1, 160); // データ配列を作成
                csv = SaveFile.CreateCsv("BveTs Vehicle Performance Table 1.00", force); // CSV形式に変換
                if(!SaveFile.SaveText(csv, filepath + "Force.csv")) // ファイル保存、キャンセル選択でメソッドを抜ける
                {
                    return;
                }
               

                // Current.csv
                current = OrigMath.CreateCurrentTable(accParam, 1, 160, drawCurve.MaxAcceleration(accParam));
                csv = SaveFile.CreateCsv("BveTs Vehicle Performance Table 1.00", current);
                if (!SaveFile.SaveText(csv, filepath + "Current.csv"))
                {
                    return;
                }

                // 応荷重あり
                if (Parameters.Original.LoadCompensating != 0)
                {
                    // ForceMax.csv
                    maxForce = OrigMath.CreateMaxForceTable(accParam, 1, 160);
                    csv = SaveFile.CreateCsv("BveTs Vehicle Performance Table 1.00", maxForce);
                    if (!SaveFile.SaveText(csv, filepath + "ForceMax.csv"))
                    {
                        return;
                    }

                    // CurrentMax.csv
                    maxCurrent = OrigMath.CreateMaxCurrentTable(current, force, maxForce);
                    csv = SaveFile.CreateCsv("BveTs Vehicle Performance Table 1.00", maxCurrent);
                    if (!SaveFile.SaveText(csv, filepath + "CurrentMax.csv"))
                    {
                        return;
                    }
                }
            }
        }

        private void saveFolderBrowserDialog_HelpRequest(object sender, EventArgs e)
        {
        }

        // 終了
        private void closeTsmi_Click(object sender, EventArgs e)
        {
            Close();
        }

        // 表示
        // エディットモードへ
        private void editModeTsmi_Click(object sender, EventArgs e)
        {
            toEditerMode();
        }

        // ビューワモードへ
        private void viewerModeTsmi_Click(object sender, EventArgs e)
        {
            toViewerMode();
        }

        private void openTraindatDialog_FileOk(object sender, CancelEventArgs e)
        {

        }

        private void openTraindatDialog_FileOk_1(object sender, CancelEventArgs e)
        {

        }

        /// <summary>
        /// 色選択ダイアログを開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void nonselectedColorTsmi_Click(object sender, EventArgs e)
        {
            ColorSetting colorDialog = new ColorSetting(colorRate, editerModeBgCol, viewerModeBgCol);
            if (colorDialog.ShowDialog() == DialogResult.OK)
            {
                colorRate = colorDialog.Rate;
                editerModeBgCol = colorDialog.EditerModeBgCol;
                viewerModeBgCol = colorDialog.ViewerModeBgCol;
            }
            colorDialog.Dispose();

            if (editModeTsmi.Checked)
            {
                drawCurve.BackGroundColor = editerModeBgCol;
            }
            else if (viewerModeTsmi.Checked)
            {
                drawCurve.BackGroundColor = viewerModeBgCol;
            }

            refleshImage();
        }

        private void openTextTsmi_Click(object sender, EventArgs e)
        {
            if (openTextDialog.ShowDialog() == DialogResult.OK)
            {
                OpenFile.OpenText(openTextDialog.FileName);
                setTrainDatParams(false);
            }
        }

        private void 設定CToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }



    }
}

