﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using Microsoft.VisualBasic.FileIO;
using Bvets5.Vehicle;
using OldBvets.TrainDat;

namespace BvetsTool
{
    // Scenario BveTs Scenario
    // Vehicle BveTs Vehicle
    // Parameters BveTs Vehicle Parameters 
    // Panel BveTs Instrument Panel
    // Sound BveTs Vehicle Sound
    // MotorNoise (No header, pass only)
    // Notch (No header, pass only)

    class OpenFile
    {
        static public double[][,] CsvFileParser(string filepath)
        {
            List<string[]> stringList = new List<string[]>(); // Listジェネリッククラス
            double[][,] table;
            int maxColumn = 0;

            try
            {
                FileInfo fi = new System.IO.FileInfo(filepath);

                if (fi.Length == 0)
                {
                    emptyOpenedDialog();
                    return null;
                }

                TextFieldParser parser = new TextFieldParser(filepath);

                // コメント記号の設定（以下の構文では不具合あり）
                // parser.CommentTokens[0] = "#";

                using (parser)
                {
                    string[] row;

                    parser.TextFieldType = FieldType.Delimited;
                    parser.SetDelimiters(","); // 区切り文字はコンマ

                    // parser.HasFieldsEnclosedInQuotes = false;
                    // parser.TrimWhiteSpace = false;

                    // 1行読み込み（ヘッダ）
                    row = parser.ReadFields();
                    string header = row[0].ToLower(); // ヘッダを小文字に変換


                    if (!header.Contains("bvets vehicle performance table"))
                    {
                        invalidFiletypeDialog(filepath, "性能曲線データ");
                        return null;
                    }


                    // 全行読み出し
                    while (!parser.EndOfData)
                    {
                        row = parser.ReadFields(); // 1行読み込み

                        // コメント行はスキップ
                        if (row[0].StartsWith("#"))
                        {
                            continue;
                        }

                        // 列数最大値を調べる
                        if (maxColumn < row.Length)
                        {
                            maxColumn = row.Length;
                        }

                        stringList.Add(row); // 1行追加
                    }
                }

                // 配列データ化
                table = new double[maxColumn - 1][,];

                for (int i = 0; i < maxColumn - 1; i++)
                {
                    table[i] = new double[stringList.Count, 2];

                    for (int j = 0; j < stringList.Count; j++)
                    {
                        double speed;
                        double value;

                        // 文字列から数値へ（数値に変換不能の場合は0をセット）
                        double.TryParse(stringList[j][0], out speed);
                        table[i][j, 0] = speed;

                        // 存在しないセルを参照した場合、強制的に0をセット
                        try
                        {
                            double.TryParse(stringList[j][i + 1], out value);
                            table[i][j, 1] = value;
                        }
                        catch
                        {
                            //table[i][j, 1] = 0;
                            readFailedDialog();
                            return null;
                        }
                    }
                }

            }
            catch
            {
                readFailedDialog();
                return null;
            }


            return table;
        }

        // テキスト(Ini)を開く
        static public void OpenText(string filepath)
        {
            string header;
            
            // ヘッダを取得
            using (StreamReader stream = new StreamReader(filepath))
            {
                header = stream.ReadLine().ToLower();
            }
            // パラメーターの場合
            if (header.Contains("bvets vehicle parameters"))
            {
                IniReader.IniParser(filepath);
            }
            else
            {
                invalidFiletypeDialog(filepath, "車両パラメーターファイル");
            }
        }


        /// <summary>
        /// Train.datの読み込み
        /// </summary>
        /// <param name="filepath">Train.datのパス</param>
        /// <param name="acp">加速力曲線パラメーター</param>
        static public void InportTraindat(string filepath, Acceleration[] acp)
        {
            try
            {
                TextFieldParser parser = new TextFieldParser(filepath);
                using (parser)
                {
                    string[] row; // 1行ごとの各項目データ格納

                    parser.TextFieldType = FieldType.Delimited;
                    parser.SetDelimiters(","); // 区切り文字はコンマ

                    // parser.HasFieldsEnclosedInQuotes = false;
                    // parser.TrimWhiteSpace = false;

                    // 1行読み込み（ヘッダ）
                    row = parser.ReadFields();
                    if (!row[0].Contains("BVE2"))
                    {
                        MessageBox.Show(filepath + "は有効なTrain.datではありません。",
                            "警告",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Exclamation);
                        return;
                    }

                    row = parser.ReadFields(); // 1行読み込み

                    // ノッチ曲線パラメーター
                    for (int i = 0; i < 8; i++)
                    {
                        row = parser.ReadFields(); // 1行読み込み

                        // TryParse
                        acp[i].ConstAccelerationStart = double.Parse(row[0]);
                        acp[i].ConstAccelerationEnd = double.Parse(row[1]);
                        acp[i].ConstPowerStart = double.Parse(row[2]);
                        acp[i].ConstPowerEnd = double.Parse(row[3]);
                        acp[i].CharacteristicCurveDegree = double.Parse(row[4]);
                    }

                    for (int i = 0; i < 36; i++)
                    {
                        row = parser.ReadFields(); // 1行読み込み
                    }

                    // 車両質量・両数
                    row = parser.ReadFields(); // 1行読み込み
                    Parameters.Dynamics.MotorcarWeight = double.Parse(row[0]) * 1000;
                    row = parser.ReadFields(); // 1行読み込み
                    Parameters.Dynamics.MotorcarCount = double.Parse(row[0]);
                    row = parser.ReadFields(); // 1行読み込み
                    Parameters.Dynamics.TrailerWeight = double.Parse(row[0]) * 1000;
                    row = parser.ReadFields(); // 1行読み込み
                    Parameters.Dynamics.TrailerCount = double.Parse(row[0]);

                    for (int i = 0; i < 9; i++)
                    {
                        row = parser.ReadFields(); // 1行読み込み
                    }

                    // 応荷重
                    row = parser.ReadFields(); // 1行読み込み
                    Parameters.Original.LoadCompensating = double.Parse(row[0]);
                }
            }
            catch
            {
                MessageBox.Show("ファイルの読み込みに失敗しました。",
                    "警告",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
                return;
            }
        }

        // ダイアログ
        private static void emptyOpenedDialog()
        {
            MessageBox.Show("ファイルが空です。",
                "警告",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation);
        }

        private static void readFailedDialog()
        {
            MessageBox.Show("ファイルの読み込みに失敗しました。",
                "警告",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation);
        }

        private static void invalidFiletypeDialog(string filepath, string filetype)
        {
            MessageBox.Show(filepath + "は有効な" + filetype + "ではありません。",
                "警告",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation);
        }
    }

    class SaveFile
    {
        /// <summary>
        /// csv形式のテキスト作成
        /// </summary>
        /// <param name="h">ヘッダーテキスト</param>
        /// <param name="table">データテーブル</param>
        /// <returns></returns>
        static public string CreateCsv(string h, double[,] table)
        {
            string csv = h;
            int a = table.GetLength(0);
            int b = table.GetLength(1);
            for (int i = 0; i < table.GetLength(1); i++)
            {
                csv += "\r\n";
                for (int j = 0; j < table.GetLength(0); j++)
                {
                    csv += ((int)table[j, i]).ToString();
                    csv += ",";
                }
            }
            return csv;
        }

        /// <summary>
        /// ファイルの保存
        /// </summary>
        /// <param name="text">保存するテキスト内容</param>
        /// <param name="filepath">保存するファイル名＋パス</param>
        /// <returns>続行ならtrue、キャンセルならfalse</returns>
        static public bool SaveText(string text, string filepath)
        {
            StreamWriter sw = null;
            DialogResult result;

            try
            {
                // ファイル存在時の上書き確認
                if (File.Exists(filepath))
                {
                    result = MessageBox.Show(filepath + "は既に存在します。上書きしますか？",
                        "ファイルの上書き確認",
                        MessageBoxButtons.YesNoCancel,
                        MessageBoxIcon.Exclamation,
                        MessageBoxDefaultButton.Button2);

                    if (result == DialogResult.Yes)
                    {
                        sw = new StreamWriter(filepath, false, Encoding.UTF8); // UTF-8で書き出す
                        sw.Write(text); // 書き出し（上書き）
                    }
                    else if (result == DialogResult.Cancel)
                    {
                        return false; // キャンセル
                    }
                }
                else
                {
                    sw = new StreamWriter(filepath, false, Encoding.UTF8); // UTF-8で書き出す
                    sw.Write(text); // 書き出し（上書き）
                }
            }
                /*
            catch (DirectoryNotFoundException ex)
            {
                // 選択ディレクトリが存在しない場合
                MessageBox.Show("選択されたディレクトリは存在しません。",
                    "警告",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
                return false;
            }
            catch (System.IO.IOException ex)
            {
                // ファイルがロックされている場合
                MessageBox.Show(filepath + "はロックされている可能性があります。",
                    "警告",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }
                 */
            catch (System.UnauthorizedAccessException ex)
            {
                // アクセス許可が無い場合
                MessageBox.Show(filepath + "へのアクセス許可がありません。",
                    "警告",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }
            finally
            {
                // 最終処理
                if (sw != null)
                    sw.Close();
            }
            return true;
        }


    }
}
